@@ -62,13 +62,14 @@ $Math_del = "\$"; #UNquoted '$'
 $MAXLOOPS = 1000;
 
 
-sub Rdconv { # Rdconv(foobar.Rd, type, debug, filename)
+sub Rdconv { # Rdconv(foobar.Rd, type, debug, filename, pkgname)
 
     $Rdname = $_[0];
     open rdfile, "<$Rdname" || die "Rdconv(): Couldn't open '$Rdfile':$!\n";
 
     $type = $_[1];
     $debug = $_[2];
+    $pkgname = $_[4];
 
     if($type !~ /,/) {
 	## Trivial (R 0.62 case): Only 1 $type at a time ==> one filename is ok.
@@ -96,9 +97,22 @@ sub Rdconv { # Rdconv(foobar.Rd, type, debug, filename)
     undef @section_body;
     undef @section_title;
 
-
+    $skipping = 0;
     #-- remove comments (everything after a %)
     while(<rdfile>){
+	if (/^#ifdef\s+([A-Za-z0-9]+)/o) {
+	    if ($1 ne $OSdir) { $skipping = 1; }
+	    next;
+	}
+	if (/^#ifndef\s+([A-Za-z0-9]+)/o) {
+	    if ($1 eq $OSdir) { $skipping = 1; }
+	    next;
+	}
+	if (/^#endif/o) {
+	    $skipping = 0;
+	    next;
+	}
+	next if $skipping > 0;
 	next if /^\s*%/o;#- completely drop full comment lines
 	my $loopcount = 0;
 	while(checkloop($loopcount++, $_, "\\%") &&
@@ -445,12 +459,13 @@ sub rdoc2html { # (filename) ; 0 for STDOUT
     if($_[0]!= -1) {
       if($_[0]) { open htmlout, "> $_[0]"; } else { open htmlout, "| cat"; }
     }
-    print htmlout html_functionhead($blocks{"title"});
+    print htmlout html_functionhead($blocks{"title"}, $pkgname,
+				    $blocks{"name"});
 
+    html_print_block("description", "Description");
     html_print_codeblock("usage", "Usage");
     html_print_argblock("arguments", "Arguments");
     html_print_block("format", "Format");
-    html_print_block("description", "Description");
     html_print_block("details", "Details");
     html_print_argblock("value", "Value");
 
@@ -591,7 +606,7 @@ sub code2html {
 	$argkey =~ s/&lt;/</go;
 	$argkey =~ s/&gt;/>/go;
 	$htmlfile = $htmlindex{$argkey};
-	
+
 	if($htmlfile){
 	    $text =~
 		s/\\link$id.*$id/<A HREF=\"..\/..\/$htmlfile\">$arg<\/A>/s;
@@ -775,13 +790,14 @@ sub rdoc2nroff { # (filename); 0 for STDOUT
     print nroffout ".pl 100i\n";
     print nroffout ".po 3\n";
     print nroffout ".na\n";
+    print nroffout ".tl '", $blocks{"name"},
+          "($pkgname)''R Documentation'\n\n" if $pkgname;
     print nroffout ".SH\n";
     print nroffout $blocks{"title"}, "\n";
-
-    nroff_print_codeblock("usage", "");
+    nroff_print_block("description", "Description");
+    nroff_print_codeblock("usage", "Usage");
     nroff_print_argblock("arguments", "Arguments");
     nroff_print_block("format", "Format");
-    nroff_print_block("description", "Description");
     nroff_print_block("details", "Details");
     nroff_print_argblock("value", "Value");
 
@@ -822,8 +838,6 @@ sub text2nroff {
     $text = nroff_tables($text);
     $text =~ s/\\cr\n?/\n.br\n/sgo;
 
-
-
     $text =~ s/\n\s*\n/\n.IP \"\" $indent\n/sgo;
     $text =~ s/\\dots/\\&.../go;
     $text =~ s/\\ldots/\\&.../go;
@@ -1256,9 +1270,9 @@ sub rdoc2ex { # (filename)
 	    }
 	    print Exout "\n";
 	}
-	
+
 	ex_print_exampleblock("examples", "Examples");
-	
+
 	if(@keywords) {
 	    print Exout "## Keywords: ";
 	    &print_vec(Exout, 'keywords');
@@ -1320,16 +1334,16 @@ sub rdoc2latex {# (filename)
       print STDERR "rdoc2l: alias='$_', code2l(.)='$c', latex_c_a(.)='$a'\n"
 	if $debug;
       printf latexout "\\alias\{%s\}\{%s\}\n", $a, $blocks{"name"}
-	unless /^$blocks{"name"}$/;
+	unless /^\Q$blocks{"name"}\E$/; # Q..E : Quote (escape) Metacharacters
     }
     foreach (@keywords) {
       printf latexout "\\keyword\{%s\}\{%s\}\n", $_, $blocks{"name"}
       unless /^$/ ;
     }
+    latex_print_block("description", "Description");
     latex_print_codeblock("usage", "Usage");
     latex_print_argblock("arguments", "Arguments");
     latex_print_block("format", "Format");
-    latex_print_block("description", "Description");
     latex_print_block("details", "Details");
     latex_print_argblock("value", "Value");
 
@@ -1524,7 +1538,7 @@ sub latex_code_cmd {
 	  if $code =~ /@/;
 	warn("\nERROR: found `HYPERLINK(' in \$code: '" . $code ."'\n")
 	  if $code =~ /HYPERLINK\(/;
-	## till 0.63.1 
+	## till 0.63.1
 	## $code = "\\verb@" . $code . "@";
 	##          [Problem: Fails in new Methods.Rd: verb NOT in command arg!
 	$code =~ s/[$LATEX_SPECIAL]/\\$&/go;# escape them (not the "bsl" )
