@@ -35,7 +35,7 @@ static char *SaveString(SEXP sxp, int offset)
     char *s;
 
     if (!isString(sxp) || length(sxp) <= offset)
-	errorcall(gcall, "invalid string argument\n");
+	errorcall(gcall, "invalid string argument");
     s = R_alloc(strlen(CHAR(STRING(sxp)[offset])) + 1, sizeof(char));
     strcpy(s, CHAR(STRING(sxp)[offset]));
     return s;
@@ -76,7 +76,7 @@ SEXP do_devga(SEXP call, SEXP op, SEXP args, SEXP env)
     GInit(&dd->dp);
     if (!X11DeviceDriver(dd, display, width, height, ps)) {
 	free(dd);
-	errorcall(call, "unable to start device devga\n");
+	errorcall(call, "unable to start device devga");
     }
     gsetVar(install(".Device"),
 	    mkString(display[0] ? display : "windows"), R_NilValue);
