@@ -41,7 +41,7 @@ HINSTANCE hUnzipDll;
 window w;
 button bBack, bNext, bFinish, bCancel, bSrc, bDest;
 radiobutton sys, pkg;
-checkbox basepkg, texthelp, htmlhelp, ltxhelp, chmhelp, winhelp, srcsp, overwrite;
+checkbox basepkg, texthelp, htmlhelp, ltxhelp, chmhelp, winhelp, srcsp, pdf, overwrite;
 listbox packages;
 textbox unzout;
 label lVer, lsrc, ldest, lwhat1, lwhat2, lwarn2, lwarn3, lwarn4, lwarn5,
@@ -55,7 +55,7 @@ field fRver, fSrc, fDest;
 int FullInstall = 1, over;
 char Rver[20]=RVER, src[MAX_PATH], dest[MAX_PATH];
 char selpkg[80], *pkglist[100], *selpkglist[100];
-int npkgs, nspkgs, ispkgs, rwb=1, rwh=1, rwch=1, rww=1, rwl=1, rwwh=0, rwsp=0;
+int npkgs, nspkgs, ispkgs, rwb=1, rwh=1, rwch=1, rww=0, rwl=0, rwwh=0, rwsp=0, rwd=0;
 int prwb=1, prww=1, prwl=1, prwch=1, prwwh=0;
 
 /* SHELLsort -- corrected from R. Sedgewick `Algorithms in C' */
@@ -169,6 +169,10 @@ void next1(button b)
 	    return;
 	}
     }
+    if(!FullInstall && strcmp("rw0900", dest + strlen(dest) - 6) == 0 ) {
+	strcat(dest, "\\library");
+    }
+    
     cleanpage1();
     if(FullInstall) page2(); else pagepkg1();
 }
@@ -197,6 +201,7 @@ void cleanpage2()
     delobj(chmhelp);
     delobj(winhelp);
     delobj(srcsp);
+    delobj(pdf);
     delobj(lwhat2);
     delobj(lwarn2);
     delobj(overwrite);
@@ -259,6 +264,7 @@ void next2(button b)
     rwch = ischecked(chmhelp);
     rwwh = ischecked(winhelp);
     rwsp = ischecked(srcsp);
+    rwd = ischecked(pdf);
     if(!rwb) {
 	strcpy(str, dest);
 	strcat(str, "/");
@@ -273,7 +279,7 @@ void next2(button b)
 	    return;
 	}
     }
-    if(!rwb & !rwh & !rww & !rwl & !rwch & !rwwh & !rwsp) return;
+    if(!rwb & !rwh & !rww & !rwl & !rwch & !rwwh & !rwsp & !rwd) return;
     over = ischecked(overwrite);
     cleanpage2();
     page3();
@@ -505,7 +511,12 @@ void page1()
     fRver = newfield(Rver, rect(180, ypos+30, 75, 20));
     pkg =   newradiobutton("An add-on package",
 			   rect(100, ypos+60, 200, 20), cPkg);
-    if(FullInstall) check(sys); else check(pkg);
+    if(FullInstall) 
+	check(sys); 
+    else {
+	check(pkg); 
+	disable(fRver);
+    }
 
     ypos = 150;
     lsrc = newlabel("Source location:", rect(10, ypos+2, 80, 20), AlignRight);
@@ -529,7 +540,7 @@ void page1()
 void page2()
 {
     int xpos = 80, ypos = 70, zips = 0;
-    char str[MAX_PATH];
+    char str[MAX_PATH], str2[MAX_PATH];
 
     clear(w); redraw(w); header();
 
@@ -537,8 +548,9 @@ void page2()
 		     rect(10, 50, 390, 20), AlignLeft);
 
     basepkg   = newcheckbox("base package", rect(xpos, ypos, 150, 20), NULL);
-    strcpy(str, Rver); strcat(str, "b.zip");
-    if(!fexists(str)) {
+    strcpy(str, Rver); strcat(str, "b1.zip");
+    strcpy(str2, Rver); strcat(str2, "b2.zip");
+    if(!fexists(str) || !fexists(str2)) {
 	uncheck(basepkg); disable(basepkg);
     } else {
 	if(rwb) check(basepkg); else uncheck(basepkg);
@@ -556,6 +568,17 @@ void page2()
 	zips++;
     }
 
+    ypos += 17;
+    chmhelp   = newcheckbox("Compiled HTML help", rect(xpos, ypos, 150, 20),
+			    NULL);
+    strcpy(str, Rver); strcat(str, "ch.zip");
+    if(!fexists(str)) {
+	uncheck(chmhelp); disable(chmhelp);
+    } else {
+	if(rwch) check(chmhelp); else uncheck(chmhelp);
+	zips++;
+    }
+
     ypos += 17;
     htmlhelp  = newcheckbox("HTML help", rect(xpos, ypos, 150, 20), NULL);
     strcpy(str, Rver); strcat(str, "w.zip");
@@ -570,21 +593,10 @@ void page2()
     ltxhelp = newcheckbox("latex help (for off-line printing)",
 			    rect(xpos, ypos, 300, 20), NULL);
     strcpy(str, Rver); strcat(str, "l.zip");
-    if(!fexists(str))
-	disable(ltxhelp);
-    else {
-	if(rwl) check(ltxhelp); else uncheck(ltxhelp);
-	zips++;
-    }
-
-    ypos += 17;
-    chmhelp   = newcheckbox("Compiled HTML help", rect(xpos, ypos, 150, 20),
-			    NULL);
-    strcpy(str, Rver); strcat(str, "ch.zip");
     if(!fexists(str)) {
-	uncheck(chmhelp); disable(chmhelp); hide(chmhelp);
+	uncheck(ltxhelp); disable(ltxhelp);
     } else {
-	if(rwch) check(chmhelp); else uncheck(chmhelp);
+	if(rwl) check(ltxhelp); else uncheck(ltxhelp);
 	zips++;
     }
 
@@ -592,7 +604,7 @@ void page2()
     winhelp   = newcheckbox("Windows help", rect(xpos, ypos, 150, 20), NULL);
     strcpy(str, Rver); strcat(str, "wh.zip");
     if(!fexists(str)) {
-	uncheck(winhelp); disable(winhelp); hide(winhelp);
+	uncheck(winhelp); disable(winhelp); /* hide(winhelp);*/
     } else {
 	if(rwwh) check(winhelp); else uncheck(winhelp);
 	zips++;
@@ -603,13 +615,24 @@ void page2()
 			  rect(xpos, ypos, 300, 20), NULL);
     strcpy(str, Rver); strcat(str, "sp.zip");
     if(!fexists(str)) {
-	uncheck(srcsp); disable(srcsp);
+	uncheck(srcsp); disable(srcsp); /* hide(srcsp);*/
     } else {
 	if(rwsp) check(srcsp); else uncheck(srcsp);
 	zips++;
     }
 
-    ypos += 30;
+    ypos += 17;
+    pdf   = newcheckbox("PDF manuals",
+			  rect(xpos, ypos, 300, 20), NULL);
+    strcpy(str, Rver); strcat(str, "d.zip");
+    if(!fexists(str)) {
+	uncheck(pdf); disable(pdf); /* hide(pdf);*/
+    } else {
+	if(rwd) check(pdf); else uncheck(pdf);
+	zips++;
+    }
+
+    ypos += 25;
     lwarn2 = newlabel("Components that are not found will be greyed out",
 		     rect(10, ypos, 390, 20), AlignLeft);
 
@@ -638,7 +661,7 @@ void page2()
 	}
     }
     overwrite = newcheckbox("overwrite existing files?",
-			    rect(30, 230, 150, 20), NULL);
+			    rect(30, 235, 150, 20), NULL);
     check(overwrite);
     setkeydown(w, key2);
     show(w);
@@ -673,8 +696,14 @@ void page3()
 	lres3 = newlabel(lab, rect(30, 240, 350, 20), AlignLeft);
 	strcpy(dest1, dest);
 	strcpy(str, src); strcat(str, "/");
-	strcat(str, Rver); strcat(str, "b.zip");
+	strcat(str, Rver); strcat(str, "b1.zip");
 	rc = do_unzip(str, dest1, 0, NULL, 0, NULL, over);
+	if(!rc) {
+	    strcpy(dest1, dest);
+	    strcpy(str, src); strcat(str, "/");
+	    strcat(str, Rver); strcat(str, "b2.zip");
+	    rc = do_unzip(str, dest1, 0, NULL, 0, NULL, over);	    
+	}
 	if(!rc) {
 	    strcpy(lab, lab2); delobj(lres3); strcat(lab, "base files  ");
 	    lres3 = newlabel(lab, rect(30, 240, 350, 20), AlignLeft);
@@ -752,6 +781,18 @@ void page3()
 	    lres3 = newlabel(lab, rect(30, 240, 350, 20), AlignLeft);
 	}
     }
+    if(rwd) {
+	strcpy(lab2, lab); delobj(lres3); strcat(lab, "PDF manuals . . .");
+	lres3 = newlabel(lab, rect(30, 240, 350, 20), AlignLeft);
+	strcpy(str, src); strcat(str, "/");
+	strcat(str, Rver); strcat(str, "d.zip");
+	strcpy(dest1, dest);
+	rc = do_unzip(str, dest1, 0, NULL, 0, NULL, over);
+	if(rc) {
+	    strcpy(lab, lab2); delobj(lres3); strcat(lab, "docs ");
+	    lres3 = newlabel(lab, rect(30, 240, 350, 20), AlignLeft);
+	}
+    }
     delobj(lres3);
     strcat(lab, "installed");
     lres3 = newlabel(lab, rect(30, 240, 350, 20), AlignLeft);
@@ -786,6 +827,7 @@ void pagepkg1()
 	while ((de = readdir(dir))) {
 	    p = de->d_name;
 	    p1 = p + strlen(p) - 4;
+	    if(!strcmp(p, "rw")) continue;
 	    if(strcmp(p1, ".zip")) continue;
 	    pkglist[npkgs] =
 		(char *)malloc((strlen(p)+1) * sizeof(char));
@@ -855,12 +897,10 @@ void pagepkg2()
     ypos += 20;
     chmhelp   = newcheckbox("compiled HTML", rect(xpos, ypos, 150, 20), NULL);
     if(prwch) check(chmhelp); else uncheck(chmhelp);
-    disable(chmhelp); hide(chmhelp);
 
     ypos += 20;
     winhelp   = newcheckbox("Windows help", rect(xpos, ypos, 150, 20), NULL);
     if(prwwh) check(winhelp); else uncheck(winhelp);
-    disable(winhelp); hide(winhelp);
 
     overwrite = newcheckbox("overwrite existing files?",
 			    rect(30, 220, 150, 20), NULL);
@@ -912,7 +952,6 @@ void pagepkg3()
 	    p = xfiles[nxfiles++] = (char*) malloc(50);
 	    strcpy(p, selpkg); strcat(p, "/latex/*");
 	}
-	if(FALSE) {
 	if(!prwch) {
 	    p = xfiles[nxfiles++] = (char*) malloc(50);
 	    strcpy(p, selpkg); strcat(p, "/chtml/*");
@@ -921,7 +960,6 @@ void pagepkg3()
 	    p = xfiles[nxfiles++] = (char*) malloc(50);
 	    strcpy(p, selpkg); strcat(p, "/winhlp/*");
 	}
-	}
     } else {
 	if(prww) {
 	    p = files[nfiles++] = (char*) malloc(50);
@@ -931,7 +969,6 @@ void pagepkg3()
 	    p = files[nfiles++] = (char*) malloc(50);
 	    strcpy(p, selpkg); strcat(p, "/latex/*");
 	}
-	if(FALSE) {
 	if(prwch) {
 	    p = files[nfiles++] = (char*) malloc(50);
 	    strcpy(p, selpkg); strcat(p, "/chtml/*");
@@ -940,7 +977,6 @@ void pagepkg3()
 	    p = files[nfiles++] = (char*) malloc(50);
 	    strcpy(p, selpkg); strcat(p, "/winhlp/*");
 	}
-	}
     }
     rc = do_unzip(zipname, dest, nfiles, files, nxfiles, xfiles, over);
     if(!rc) {
@@ -950,7 +986,12 @@ void pagepkg3()
 	lresp2 = newlabel(cmd, rect(50, 220, 300, 20), Center);
     }
     enable(bBack); 
-    if(ispkgs < nspkgs) enable(bNext); else enable(bFinish);
+    if(ispkgs < nspkgs) 
+	enable(bNext); 
+    else {
+	enable(bFinish);
+	if(prww) askok("You will need to run\n\nlink.html.help()\n\nin R to update the\npackages list for help.start()");
+    }
     setkeydown(w, keypkg3);
     show(w);
 }
