@@ -176,22 +176,41 @@ typedef struct RCNTXT {
     void (*cend)();		/* C "on.exit" thunk */
 } RCNTXT, *context;
 
-/* The Various Context Types.  In general the type is a */
-/* bitwise OR of the values below.  Note that CTXT_LOOP */
-/* is already the or of CTXT_NEXT and CTXT_BREAK. */
+/* The Various Context Types.
+
+ * In general the type is a bitwise OR of the values below.
+ * Note that CTXT_LOOP is already the or of CTXT_NEXT and CTXT_BREAK.
+ * Only functions should have the third bit turned on;
+ * this allows us to move up the context stack easily
+ * with either RETURN's or GENERIC's or RESTART's.
+ * If you add a new context type for functions make sure
+ *   CTXT_NEWTYPE & CTXT_FUNCTION > 0
+ */
 enum {
     CTXT_TOPLEVEL = 0,
     CTXT_NEXT	  = 1,
     CTXT_BREAK	  = 2,
     CTXT_LOOP	  = 3,	/* break OR next target */
-    CTXT_RETURN	  = 4,
+    CTXT_FUNCTION = 4,
     CTXT_CCODE	  = 8,
-    CTXT_BROWSER  = 12,
-    CTXT_GENERIC  = 16,
-    CTXT_RESTART  = 36
+    CTXT_RETURN	  = 12,
+    CTXT_BROWSER  = 16,
+    CTXT_GENERIC  = 20,
+    CTXT_RESTART  = 28
 };
 
-
+/*
+TOP   0 0 0 0 0 0  = 0
+NEX   1 0 0 0 0 0  = 1
+BRE   0 1 0 0 0 0  = 2
+LOO   1 1 0 0 0 0  = 3
+FUN   0 0 1 0 0 0  = 4
+CCO   0 0 0 1 0 0  = 8
+BRO   0 0 0 0 1 0  = 16
+RET   0 0 1 1 0 0  = 12
+GEN   0 0 1 0 1 0  = 20
+RES   0 0 1 1 1 0  = 28
+*/
 /* Miscellaneous Definitions */
 #define streql(s, t)	(!strcmp((s), (t)))
 
@@ -212,7 +231,10 @@ enum {
 #define	GTOP	6
 
 /* File Handling */
+/*
 #define R_EOF	65535
+*/
+#define R_EOF	-1
 
 /* MAGIC Numbers for files */
 #define R_MAGIC_BINARY 1975
@@ -251,7 +273,7 @@ FUNTAB	R_FunTab[];	    /* Built in functions */
 #endif
 
 extern int	errno;
-extern int	gc_inhibit_torture;
+extern int	gc_inhibit_torture INI_as(1);
 
 /* R Home Directory */
 extern char*	R_Home;		    /* Root of the R tree */
@@ -287,7 +309,6 @@ extern int	R_BrowseLevel	INI_as(0);	/* how deep the browser is */
 
 /* File Input/Output */
 extern int	R_Interactive	INI_as(1);	/* Non-zero during interactive use */
-extern int	R_Error_Halt	INI_as(1);	/* For non-interactive.. */
 extern int	R_Quiet		INI_as(0);	/* Be as quiet as possible */
 extern int	R_Slave		INI_as(0);	/* Run as a slave process */
 extern int	R_Verbose	INI_as(0);	/* Be verbose */
@@ -344,7 +365,7 @@ void	R_ResetConsole(void);
 void	R_FlushConsole(void);
 void	R_ClearerrConsole(void);
 void	R_Busy(int);
-void	R_CleanUp(int);
+void	R_CleanUp(int, int, int);
 void	R_StartUp(void);
 int	R_ShowFile(char*, char*);
 int	R_ShowFiles(int, char **, char **, char *, int, char *);
