@@ -1,20 +1,32 @@
 ## Copyright (C) 1998 John W. Emerson
 
+
 mosaicplot <- function(x, ...) UseMethod("mosaicplot")
 
-mosaicplot.default <- function(X, main = NA, sort = NA, off = NA, dir =
-                       NA, color = FALSE) {
+### Changes by MM:
+## - NULL instead of NA for default arguments, etc  [R / S convention]
+## - plotting at end; cosmetic
+## - mosaic.cell():
 
-    mosaic.cell <- function(X, x1, y1, x2, y2, off, dir, color, lablevx,
-                            lablevy, maxdim, currlev, label) {
+mosaicplot.default <- function(X, main = NULL, xlab = NULL, ylab = NULL,
+                               sort = NULL, off = NULL,
+                               dir = NULL, color = FALSE) {
 
+    mosaic.cell <- function(X, x1, y1, x2, y2,
+                            off, dir, color, lablevx, lablevy,
+                            maxdim, currlev, label)
+    {
+        ## Recursive function doing `the job'
+        ##
+        ## explicitely relying on (1,1000)^2 user coordinates.
+        p <- ncol(X)
         if (dir[1] == "v") {            # split here on the X-axis.
             xdim <- maxdim[1]
             XP <- rep(0, xdim)
             for (i in 1:xdim) {
-                XP[i] <- sum(X[X[,1]==i,ncol(X)]) / sum(X[,ncol(X)])
+                XP[i] <- sum(X[X[,1]==i,p]) / sum(X[,p])
             }
-            white <- off[1] * (x2 - x1) / (max(1, xdim-1))
+            white <- off[1] * (x2 - x1) / max(1, xdim-1)
             x.l <- x1
             x.r <- x1 + (1 - off[1]) * XP[1] * (x2 - x1)
             if (xdim > 1) {
@@ -25,31 +37,32 @@ mosaicplot.default <- function(X, main = NA, sort = NA, off = NA, dir =
                 }
             }
             if (lablevx > 0) {
-                if (is.na(label[[1]][1])) {
-                    this.lab <- paste(rep(as.character(currlev),
-                                          length(currlev)),
-                                      as.character(1:xdim), sep=".")
-                } else { this.lab <- label[[1]] }
-                text(x=(x.l + (x.r - x.l) / 2),
-                     y=(965 + 22 * (lablevx - 1)), 
-                     srt=0,adj=.5, cex=.5, this.lab)
+                this.lab <-
+                    if (is.null(label[[1]][1])) {
+                        paste(rep(as.character(currlev),
+                                  length(currlev)),
+                              as.character(1:xdim), sep=".")
+                    } else label[[1]]
+                text(x= x.l + (x.r - x.l) / 2,
+                     y= 965 + 22 * (lablevx - 1),
+                     srt=0, adj=.5, cex=.5, this.lab)
             }
-            if (ncol(X) > 2) {          # recursive call.
+            if (p > 2) {          # recursive call.
                 for (i in 1:xdim) {
                     if (XP[i] > 0) {
-                        mosaic.cell(as.matrix(X[X[,1]==i,2:ncol(X)]),
+                        mosaic.cell(as.matrix(X[X[,1]==i, 2:p]),
                                     x.l[i], y1, x.r[i], y2,
                                     off[2:length(off)],
                                     dir[2:length(dir)],
                                     color, lablevx-1, (i==1)*lablevy,
-                                    maxdim[2:length(maxdim)], 
-                                    currlev+1, label[2:ncol(X)])
+                                    maxdim[2:length(maxdim)],
+                                    currlev+1, label[2:p])
                     } else {
                         segments(rep(x.l[i],3), y1+(y2-y1)*c(0,2,4)/5,
                                  rep(x.l[i],3), y1+(y2-y1)*c(1,3,5)/5)
                     }
                 }
-            } else {
+            } else { # ncol(X) <= 1 : final split polygon and segments.
                 for (i in 1:xdim) {
                     if (XP[i] > 0) {
                         polygon(c(x.l[i], x.r[i], x.r[i], x.l[i]),
@@ -64,11 +77,11 @@ mosaicplot.default <- function(X, main = NA, sort = NA, off = NA, dir =
                     }
                 }
             }
-        } else {                        # split here on the Y-axis.
+        } else { ## dir[1] - "horizontal" : split here on the Y-axis.
             ydim <- maxdim[1]
             YP <- rep(0, ydim)
             for (j in 1:ydim) {
-                YP[j] <- sum(X[X[,1]==j,ncol(X)]) / sum(X[,ncol(X)])
+                YP[j] <- sum(X[X[,1]==j,p]) / sum(X[,p])
             }
             white <- off[1] * (y2 - y1) / (max(1, ydim - 1))
             y.b <- y2 - (1 - off[1]) * YP[1] * (y2 - y1)
@@ -81,38 +94,38 @@ mosaicplot.default <- function(X, main = NA, sort = NA, off = NA, dir =
                 }
             }
             if (lablevy > 0) {
-                if (is.na(label[[1]][1])) {
-                    this.lab <- paste(rep(as.character(currlev),
-                                          length(currlev)), 
-                                      as.character(1:ydim), sep=".")
-                } else { this.lab <- label[[1]] }
-                text(x=(35 - 20 * (lablevy - 1)),
-                     y=(y.b + (y.t - y.b) / 2),
+                this.lab <-
+                    if (is.null(label[[1]][1])) {
+                        paste(rep(as.character(currlev),
+                                  length(currlev)),
+                              as.character(1:ydim), sep=".")
+                    } else label[[1]]
+                text(x= 35 - 20 * (lablevy - 1),
+                     y= y.b + (y.t - y.b) / 2,
                      srt=90, adj=.5, cex=.5, this.lab)
             }
-            if (ncol(X) > 2) {          # recursive call.
+            if (p > 2) {          # recursive call.
                 for (j in 1:ydim) {
                     if (YP[j] > 0) {
-                        mosaic.cell(as.matrix(X[X[,1]==j,2:ncol(X)]),
+                        mosaic.cell(as.matrix(X[X[,1]==j,2:p]),
                                     x1, y.b[j], x2, y.t[j],
                                     off[2:length(off)],
                                     dir[2:length(dir)], color,
                                     (j==1)*lablevx, lablevy-1,
-                                    maxdim[2:length(maxdim)], 
-                                    currlev+1, label[2:ncol(X)])
+                                    maxdim[2:length(maxdim)],
+                                    currlev+1, label[2:p])
                     } else {
                         segments(x1+(x2-x1)*c(0,2,4)/5, rep(y.b[j],3),
                                  x1+(x2-x1)*c(1,3,5)/5, rep(y.b[j],3))
                     }
                 }
-            } else{                     # final split polygon and segments.
+            } else {  # ncol(X) <= 1: final split polygon and segments.
                 for (j in 1:ydim) {
                     if (YP[j] > 0) {
                         polygon(c(x1,x2,x2,x1),
-                                c(y.b[j],y.b[j],y.t[j],y.t[j]),
-                                col=color[j])
+                                c(y.b[j],y.b[j],y.t[j],y.t[j]), col=color[j])
                         segments(c(x1,x1,x1,x2),
-                                 c(y.b[j],y.b[j],y.t[j],y.t[j]), 
+                                 c(y.b[j],y.b[j],y.t[j],y.t[j]),
                                  c(x2,x1,x2,x2),
                                  c(y.b[j],y.t[j],y.t[j],y.b[j]))
                     } else {
@@ -124,50 +137,66 @@ mosaicplot.default <- function(X, main = NA, sort = NA, off = NA, dir =
         }
     }
 
-    frame()
-    opar <- par(usr = c(1,1000,1,1000))
-    on.exit(par(opar))
-    if (is.vector(X)) { X <- array(X) }
-    dimd <- length(dim(X))
-    if (!is.null(dimnames(X))) { label <- dimnames(X) } else { label <- NA }
-    if (dimd>1) {
-        Ind <- rep(1:(dim(X)[1]), prod(dim(X)[2:dimd]))
+    ##-- Begin main function
+    if(is.null(dim(X)))
+        X <- as.array(X)
+    else if(is.data.frame(X))
+        X <- data.matrix(X)
+    dimd <- length(dX <- dim(X))
+    if(dimd == 0 || any(dX == 0))
+        stop("`X' must not have 0 dimensionality")
+    ##-- Set up `Ind' matrix : to contain indices and data
+    Ind <- 1:dX[1]
+    if(dimd > 1) {
+        Ind <- rep(Ind, prod(dX[2:dimd]))
         for (i in 2:dimd) {
             Ind <- cbind(Ind,
-                         c(matrix(1:(dim(X)[i]), byrow=TRUE,
-                                  prod(dim(X)[1:(i-1)]),
-                                  prod(dim(X)[i:dimd]))))
+                         c(matrix(1:dX[i], byrow=TRUE,
+                                  nr = prod(dX[1:(i-1)]),
+                                  nc = prod(dX[i:dimd]))))
         }
-    } else {
-        Ind <- 1:(dim(X)[1])
     }
     Ind <- cbind(Ind, c(X))
-    if (!is.na(main)) { title(main) }   # Make the title.
-    if ((is.na(off[1]))||(length(off)!=dimd)) { # Initialize spacing.
-        off <- rep(10,50)[1:dimd]
+    ## The next four may all be NULL:
+    label <- dimnames(X)
+    nam.dn <- names(label)
+    if(is.null(xlab)) xlab <- nam.dn[1]
+    if(is.null(ylab)) ylab <- nam.dn[2]
+
+    if (is.null(off) || length(off) != dimd) { # Initialize spacing.
+        off <- rep(10, length=dimd)
     }
-    if (is.na(dir[1])||(length(dir)!=dimd)) { # Initialize directions.
-        dir <- rep(c("v","h"),50)[1:dimd]
+    if (is.null(dir) || length(dir) != dimd) {# Initialize directions
+        dir <- rep(c("v","h"), length=dimd)
     }
-    if ((!is.na(sort[1]))&&(length(sort)==dimd)) { # Sort columns.
+    if (!is.null(sort)) {
+        if(length(sort) != dimd)
+            stop("length(sort) doesn't conform to dim(X)")
+        ## Sort columns.
         Ind <- Ind[,c(sort,dimd+1)]
         off <- off[sort]
         dir <- dir[sort]
         label <- label[sort]
     }
     ncolors <- length(tabulate(Ind[,dimd]))
-    if (is.na(color[1])) {
-        color <- rep(0, ncolors)
-    } else {
-        if (length(color) != ncolors) {
-            if (!color[1]) { color <- rep(0, ncolors) }
-            else { color <- 2:(ncolors+1) }
-        }
-    }
+    if (is.null(color) || length(color) != ncolors)
+        color <- if (is.null(color) || !color[1])
+            rep(0, ncolors) else 2:(ncolors+1)
 
-    mosaic.cell(Ind, 50, 5, 950, 950,
-        off/100, dir, color, 2, 2, apply(as.matrix(Ind[,1:dimd]), 2, max),
-        1, label)
+    ##-- Plotting
+    frame()
+    opar <- par(usr = c(1,1000,1,1000))
+    on.exit(par(opar))
+
+    if (!is.null(main) || !is.null(xlab) || !is.null(ylab))
+        title(main, xlab=xlab, ylab=ylab)
+
+    mosaic.cell(Ind,
+                x1=50, y1=5, x2=950, y2=950,
+                off/100, dir,
+                color, 2, 2,
+                maxdim= apply(as.matrix(Ind[,1:dimd]), 2, max),
+                currlev= 1, label)
 
 }
 
