@@ -2,8 +2,7 @@
 {
     library.dynam("ts", pkg, lib)
     if(interactive() || .Options$ verbose)
-	cat("\n	     This is a preliminary time series package for R 0.65\n",
+	cat("\n	   This is a preliminary time series package for R\n",
 	    "	   See `library(help=ts)' for details.\n\n")
 }
-#.conflicts.OK <- TRUE
 options(ts.S.compat = FALSE)
