@@ -42,6 +42,7 @@ static SEXP ans_ptr;
 static int  ans_length;
 static SEXP ans_names;
 static int  ans_nnames;
+static int  deparse_level = 1;
 
 static int HasNames(SEXP x)
 {
@@ -694,7 +695,7 @@ SEXP do_unlist(SEXP call, SEXP op, SEXP args, SEXP env)
     else {
 	UNPROTECT(1);
 	if (isVector(args)) return args;
-	else errorcall(call, "argument not a list\n");
+	else errorcall(call, "argument not a list");
     }
 
     /* If a non-vector argument was encountered (perhaps a list if */
@@ -804,7 +805,7 @@ SEXP FetchMethod(char *generic, char *classname, SEXP env)
     char buf[LNAMBUF];
     SEXP method;
     if (strlen(generic) + strlen(classname) + 2 > LNAMBUF)
-	error("class name too long in %s\n", generic);
+	error("class name too long in %s", generic);
     sprintf(buf, "%s.%s", generic, classname);
     method = findVar(install(buf), env);
     if (TYPEOF(method) != CLOSXP)
@@ -827,7 +828,7 @@ SEXP do_bind(SEXP call, SEXP op, SEXP args, SEXP env)
      * is still in a position to use "substitute" to get the actual
      * expressions which generated the argument (for naming purposes).
      *
-     * The dispatch rule here is as follows: 
+     * The dispatch rule here is as follows:
      *
      * 1) For each argument we get the list of possible class
      *    memberships from the class attribute.
@@ -885,8 +886,10 @@ SEXP do_bind(SEXP call, SEXP op, SEXP args, SEXP env)
     /* Despatch based on class membership has failed. */
     /* The default code for rbind/cbind.default follows */
     /* First, extract the evaluated arguments. */
+#ifdef OLD
     for (a = args; a != R_NilValue; a = CDR(a))
         CAR(a) = PRVALUE(CAR(a));
+#endif
 
     rho = env; /* GLOBAL */
     ans_flags = 0;
@@ -894,7 +897,7 @@ SEXP do_bind(SEXP call, SEXP op, SEXP args, SEXP env)
     ans_nnames = 0;
 
     for (t = args; t != R_NilValue; t = CDR(t))
-	AnswerType(CAR(t), 0, 0);
+	AnswerType(PRVALUE(CAR(t)), 0, 0);
 
     /* zero-extent matrices shouldn't give NULL, anymore :
        if (ans_length == 0)
@@ -924,7 +927,7 @@ SEXP do_bind(SEXP call, SEXP op, SEXP args, SEXP env)
     case STRSXP:
 	break;
     default:
-	errorcall(call, "cannot create a matrix from these types\n");
+	errorcall(call, "cannot create a matrix from these types");
     }
 
     if (PRIMVAL(op) == 1)
@@ -959,7 +962,7 @@ static SEXP cbind(SEXP call, SEXP args, SEXPTYPE mode)
     int nnames, mnames;
     int rows, cols, mrows;
     int warned;
-    SEXP dn, t, u, result, dims;
+    SEXP dn, t, u, result, dims, expr;
 
     have_rnames = 0;
     have_cnames = 0;
@@ -974,17 +977,18 @@ static SEXP cbind(SEXP call, SEXP args, SEXPTYPE mode)
 
     n = 0;
     for (t = args; t != R_NilValue; t = CDR(t)) {
-	if (length(CAR(t)) >= 0) {
-	    dims = getAttrib(CAR(t), R_DimSymbol);
+	u = PRVALUE(CAR(t));
+	if (length(u) >= 0) {
+	    dims = getAttrib(u, R_DimSymbol);
 	    if (length(dims) == 2) {
 		if (mrows == -1)
 		    mrows = INTEGER(dims)[0];
 		else if (mrows != INTEGER(dims)[0])
-		    errorcall(call, "number of rows of matrices must match (see arg %d)\n", n + 1);
+		    errorcall(call, "number of rows of matrices must match (see arg %d)", n + 1);
 		cols += INTEGER(dims)[1];
 	    }
-	    else if (length(CAR(t))>0) {
-		rows = imax2(rows, length(CAR(t)));
+	    else if (length(u)>0) {
+		rows = imax2(rows, length(u));
 		cols += 1;
 	    }
 	}
@@ -997,11 +1001,12 @@ static SEXP cbind(SEXP call, SEXP args, SEXPTYPE mode)
     n = 0;
     warned = 0;
     for (t = args; t != R_NilValue; t = CDR(t)) {
+	u = PRVALUE(CAR(t));
 	n++;
-	if (length(CAR(t)) >= 0) {
-	    dims = getAttrib(CAR(t), R_DimSymbol);
+	if (length(u) >= 0) {
+	    dims = getAttrib(u, R_DimSymbol);
 	    if (length(dims) == 2) {
-		dn = getAttrib(CAR(t), R_DimNamesSymbol);
+		dn = getAttrib(u, R_DimNamesSymbol);
 		if (length(dn) == 2) {
 		    if (VECTOR(dn)[1] != R_NilValue)
 			have_cnames = 1;
@@ -1010,15 +1015,17 @@ static SEXP cbind(SEXP call, SEXP args, SEXPTYPE mode)
 		}
 	    }
 	    else {
-		k = length(CAR(t));
+		k = length(u);
 		if (!warned && k>0 && (k > rows || rows % k)) {
 		    warned = 1;
 		    PROTECT(call = substituteList(call, rho));
 		    warningcall(call, "number of rows of result\n\tis not a multiple of vector length (arg %d)", n);
 		    UNPROTECT(1);
 		}
-		dn = getAttrib(CAR(t), R_NamesSymbol);
-		if (TAG(t) != R_NilValue)
+		dn = getAttrib(u, R_NamesSymbol);
+		if (k && (TAG(t) != R_NilValue || 
+		    ((deparse_level == 1) && 
+		     isSymbol(substitute(CAR(t),R_NilValue)))))
 		    have_cnames = 1;
 		nnames = imax2(nnames, length(dn));
 	    }
@@ -1032,10 +1039,11 @@ static SEXP cbind(SEXP call, SEXP args, SEXPTYPE mode)
 
     if (mode == STRSXP) {
 	for (t = args; t != R_NilValue; t = CDR(t)) {
-	    if (length(CAR(t)) > 0) {/* rbind() has ">=" here */
-		u = CAR(t) = coerceVector(CAR(t), STRSXP);
+	    u = PRVALUE(CAR(t));
+	    if (length(u) > 0) {/* rbind() has ">=" here */
+		u = coerceVector(u, STRSXP);
 		k = LENGTH(u);
-		idx = (!isMatrix(CAR(t))) ? rows : k;
+		idx = (!isMatrix(u)) ? rows : k;
 		for (i = 0; i < idx; i++)
 		    STRING(result)[n++] = STRING(u)[i % k];
 	    }
@@ -1043,10 +1051,11 @@ static SEXP cbind(SEXP call, SEXP args, SEXPTYPE mode)
     }
     else if (mode == CPLXSXP) {
 	for (t = args; t != R_NilValue; t = CDR(t)) {
-	    if (length(CAR(t)) > 0) {
-		u = CAR(t) = coerceVector(CAR(t), CPLXSXP);
+	    u = PRVALUE(CAR(t));
+	    if (length(u) > 0) {
+		u = coerceVector(u, CPLXSXP);
 		k = LENGTH(u);
-		idx = (!isMatrix(CAR(t))) ? rows : k;
+		idx = (!isMatrix(u)) ? rows : k;
 		for (i = 0; i < idx; i++)
 		    COMPLEX(result)[n++] = COMPLEX(u)[i % k];
 	    }
@@ -1054,10 +1063,10 @@ static SEXP cbind(SEXP call, SEXP args, SEXPTYPE mode)
     }
     else {
 	for (t = args; t != R_NilValue; t = CDR(t)) {
-	    if (length(CAR(t)) > 0) {
-		u = CAR(t);
+	    u = PRVALUE(CAR(t));
+	    if (length(u) > 0) {
 		k = LENGTH(u);
-		idx = (!isMatrix(CAR(t))) ? rows : k;
+		idx = (!isMatrix(u)) ? rows : k;
 		if (TYPEOF(u) <= INTSXP) {
 		    if (mode <= INTSXP) {
 			for (i = 0; i < idx; i++)
@@ -1086,11 +1095,12 @@ static SEXP cbind(SEXP call, SEXP args, SEXPTYPE mode)
 	    nam = R_NilValue;	/* -Wall */
 	j = 0;
 	for (t = args; t != R_NilValue; t = CDR(t)) {
-	    if (length(CAR(t)) >= 0) {
+	    u = PRVALUE(CAR(t));
+	    if (length(u) >= 0) {
 
-		if (isMatrix(CAR(t))) {
+		if (isMatrix(u)) {
 
-		    u = getAttrib(CAR(t), R_DimNamesSymbol);
+		    u = getAttrib(u, R_DimNamesSymbol);
 		    tnam = GetColNames(u);
 
 		    if (have_rnames &&
@@ -1104,24 +1114,26 @@ static SEXP cbind(SEXP call, SEXP args, SEXPTYPE mode)
 			    STRING(nam)[j++] = STRING(tnam)[i];
 		    }
 		    else if (have_cnames) {
-			for (i = 0; i < ncols(CAR(t)); i++)
+			for (i = 0; i < ncols(u); i++)
 			    STRING(nam)[j++] = R_BlankString;
 		    }
 		}
-		else if (length(CAR(t)) > 0) {
+		else if (length(u) > 0) {
 
-		    u = getAttrib(CAR(t), R_NamesSymbol);
+		    u = getAttrib(u, R_NamesSymbol);
 		    tnam = u;
 
 		    if (have_rnames && GetRowNames(dn) == R_NilValue
 			&& u != R_NilValue && length(u) == rows)
 			SetRowNames(dn, duplicate(u));
 
-		    if (TAG(t) != R_NilValue) {
+		    if (TAG(t) != R_NilValue) 
 			STRING(nam)[j++] = PRINTNAME(TAG(t));
-		    } else if (have_cnames) {
+		    else if ((deparse_level == 1) && 
+			     isSymbol(expr = substitute(CAR(t), R_NilValue)))
+			STRING(nam)[j++] = PRINTNAME(expr);
+		    else if (have_cnames) 
 			STRING(nam)[j++] = R_BlankString;
-		    }
 		}
 	    }
 	}
@@ -1139,7 +1151,7 @@ static SEXP rbind(SEXP call, SEXP args, SEXPTYPE mode)
     int nnames, mnames;
     int rows, cols, mcols, mrows;
     int warned;
-    SEXP dn, t, u, result, dims;
+    SEXP dn, t, u, result, dims, expr;
 
     have_rnames = 0;
     have_cnames = 0;
@@ -1153,17 +1165,18 @@ static SEXP rbind(SEXP call, SEXP args, SEXPTYPE mode)
 
     n = 0;
     for (t = args; t != R_NilValue; t = CDR(t)) {
-	if (length(CAR(t)) >= 0) {
-	    dims = getAttrib(CAR(t), R_DimSymbol);
+	u = PRVALUE(CAR(t));
+	if (length(u) >= 0) {
+	    dims = getAttrib(u, R_DimSymbol);
 	    if (length(dims) == 2) {
 		if (mcols == 0)
 		    mcols = INTEGER(dims)[1];
 		else if (mcols != INTEGER(dims)[1])
-		    errorcall(call, "number of columns of matrices must match (see arg %d)\n", n + 1);
+		    errorcall(call, "number of columns of matrices must match (see arg %d)", n + 1);
 		rows += INTEGER(dims)[0];
 	    }
-	    else if (length(CAR(t))>0){
-		cols = imax2(cols, length(CAR(t)));
+	    else if (length(u)>0){
+		cols = imax2(cols, length(u));
 		rows += 1;
 	    }
 	}
@@ -1176,11 +1189,12 @@ static SEXP rbind(SEXP call, SEXP args, SEXPTYPE mode)
     n = 0;
     warned = 0;
     for (t = args; t != R_NilValue; t = CDR(t)) {
+	u = PRVALUE(CAR(t));
 	n++;
-	if (length(CAR(t)) >= 0) {
-	    dims = getAttrib(CAR(t), R_DimSymbol);
+	if (length(u) >= 0) {
+	    dims = getAttrib(u, R_DimSymbol);
 	    if (length(dims) == 2) {
-		dn = getAttrib(CAR(t), R_DimNamesSymbol);
+		dn = getAttrib(u, R_DimNamesSymbol);
 		if (length(dn) == 2) {
 		    if (VECTOR(dn)[0] != R_NilValue)
 			have_rnames = 1;
@@ -1189,15 +1203,17 @@ static SEXP rbind(SEXP call, SEXP args, SEXPTYPE mode)
 		}
 	    }
 	    else {
-		k = length(CAR(t));
+		k = length(u);
 		if (!warned && k>0 && (k > cols || cols % k)) {
 		    warned = 1;
 		    PROTECT(call = substituteList(call, rho));
 		    warningcall(call, "number of columns of result\n\tnot a multiple of vector length (arg %d)", n);
 		    UNPROTECT(1);
 		}
-		dn = getAttrib(CAR(t), R_NamesSymbol);
-		if (TAG(t) != R_NilValue)
+		dn = getAttrib(u, R_NamesSymbol);
+		if (k && (TAG(t) != R_NilValue || 
+		    ((deparse_level == 1) && 
+		     isSymbol(substitute(CAR(t),R_NilValue)))))
 		    have_rnames = 1;
 		nnames = imax2(nnames, length(dn));
 	    }
@@ -1211,8 +1227,9 @@ static SEXP rbind(SEXP call, SEXP args, SEXPTYPE mode)
 
     if (mode == STRSXP) {
 	for (t = args; t != R_NilValue; t = CDR(t)) {
-	    if (length(CAR(t)) >= 0) {/* cbind() has ">" here */
-		u = CAR(t) = coerceVector(CAR(t), STRSXP);
+	    u = PRVALUE(CAR(t));
+	    if (length(u) >= 0) {/* cbind() has ">" here */
+		u = coerceVector(u, STRSXP);
 		k = LENGTH(u);
 		mrows = (isMatrix(u)) ? nrows(u) : 1;
 		if (k == 0)
@@ -1228,8 +1245,9 @@ static SEXP rbind(SEXP call, SEXP args, SEXPTYPE mode)
     }
     else if (mode == CPLXSXP) {
 	for (t = args; t != R_NilValue; t = CDR(t)) {
-	    if (length(CAR(t)) > 0) {
-		u = CAR(t) = coerceVector(CAR(t), CPLXSXP);
+	    u = PRVALUE(CAR(t));
+	    if (length(u) > 0) {
+		u = coerceVector(u, CPLXSXP);
 		k = LENGTH(u);
 		mrows = (isMatrix(u)) ? nrows(u) : 1;
 		for (i = 0; i < mrows; i++)
@@ -1243,8 +1261,8 @@ static SEXP rbind(SEXP call, SEXP args, SEXPTYPE mode)
     }
     else {
 	for (t = args; t != R_NilValue; t = CDR(t)) {
-	    if (length(CAR(t))>=0) {
-		u = CAR(t);
+	    u = PRVALUE(CAR(t));
+	    if (length(u)>=0) {
 		k = LENGTH(u);
 		/* mrows = (isMatrix(u)) ? nrows(u) : 1; */
 		if (isMatrix(u)) {
@@ -1289,11 +1307,12 @@ static SEXP rbind(SEXP call, SEXP args, SEXPTYPE mode)
 	    nam = R_NilValue;	/* -Wall */
 	j = 0;
 	for (t = args; t != R_NilValue; t = CDR(t)) {
-	    if (length(CAR(t)) >= 0) {
+	    u = PRVALUE(CAR(t));
+	    if (length(u) >= 0) {
 
-		if (isMatrix(CAR(t))) {
+		if (isMatrix(u)) {
 
-		    u = getAttrib(CAR(t), R_DimNamesSymbol);
+		    u = getAttrib(u, R_DimNamesSymbol);
 		    tnam = GetRowNames(u);
 
 		    if (have_cnames && GetColNames(dn) == R_NilValue
@@ -1307,25 +1326,27 @@ static SEXP rbind(SEXP call, SEXP args, SEXPTYPE mode)
 				STRING(nam)[j++] = STRING(tnam)[i];
 			}
 			else {
-			    for (i = 0; i < nrows(CAR(t)); i++)
+			    for (i = 0; i < nrows(u); i++)
 				STRING(nam)[j++] = R_BlankString;
 			}
 		    }
 		}
-		else if (length(CAR(t)) > 0) {
+		else if (length(u) > 0) {
 
-		    u = getAttrib(CAR(t), R_NamesSymbol);
+		    u = getAttrib(u, R_NamesSymbol);
 		    tnam = u;
 
 		    if (have_cnames && GetColNames(dn) == R_NilValue
 			&& u != R_NilValue && length(u) == cols)
 			SetColNames(dn, duplicate(u));
 
-		    if (TAG(t) != R_NilValue) {
+		    if (TAG(t) != R_NilValue) 
 			STRING(nam)[j++] = PRINTNAME(TAG(t));
-		    } else if (have_rnames) {
+		    else if ((deparse_level == 1) && 
+			     isSymbol(expr = substitute(CAR(t), R_NilValue)))
+			STRING(nam)[j++] = PRINTNAME(expr);
+		    else if (have_rnames)
 			STRING(nam)[j++] = R_BlankString;
-		    }
 		}
 	    }
 	}
@@ -1335,3 +1356,5 @@ static SEXP rbind(SEXP call, SEXP args, SEXPTYPE mode)
     UNPROTECT(1);
     return result;
 } /* rbind */
+
+
