@@ -92,7 +92,7 @@ static int isUminus(SEXP s)
 		return 1;
 	    else return 0;
 	default:
-	    error("invalid form in unary minus check\n");
+	    error("invalid form in unary minus check");
 	    return -1;/* for -Wall */
 	}
     }
@@ -424,7 +424,7 @@ static SEXP D(SEXP expr, SEXP var)
 	    ans = D(expr1, var);
 	    UNPROTECT(1);
 	}
-	else error("Function %s is not in the derivatives table\n",
+	else error("Function %s is not in the derivatives table",
 		   PRINTNAME(CAR(expr)));
 	break;
     default:
@@ -526,7 +526,7 @@ SEXP do_D(SEXP call, SEXP op, SEXP args, SEXP env)
     else expr = CAR(args);
     var = CADR(args);
     if (!isString(var) || length(var) < 1)
-	errorcall(call, "variable must be a character string\n");
+	errorcall(call, "variable must be a character string");
     var = install(CHAR(STRING(var)[0]));
     InitDerivSymbols();
     PROTECT(expr = D(expr, var));
@@ -539,7 +539,7 @@ SEXP do_D(SEXP call, SEXP op, SEXP args, SEXP env)
 
 static void InvalidExpression(char *where)
 {
-    error("invalid expression in \"%s\"\n", where);
+    error("invalid expression in \"%s\"", where);
 }
 
 static int equal(SEXP expr1, SEXP expr2)
@@ -754,14 +754,14 @@ SEXP do_deriv(SEXP call, SEXP op, SEXP args, SEXP env)
     args = CDR(args);
     names = CAR(args);
     if (!isString(names) || (nderiv = length(names)) < 1)
-	errorcall(call, "invalid variable names\n");
+	errorcall(call, "invalid variable names");
     args = CDR(args);
     PROTECT(funarg = duplicate(CAR(args)));
     args = CDR(args);
     tag = CAR(args);
     if (!isString(tag) || length(tag) < 1
 	|| length(STRING(tag)[0]) < 1 || length(STRING(tag)[0]) > 60)
-	errorcall(call, "invalid tag\n");
+	errorcall(call, "invalid tag");
     /* NOTE: FindSubexprs is destructive, hence the duplication */
     PROTECT(ans = duplicate(expr));
     f_index = FindSubexprs(ans);
