@@ -283,6 +283,7 @@ FUNTAB R_FunTab[] =
 {"Conj",	do_cmathfuns,	5,	1,	1,	PP_FUNCALL},
 {"abs",		do_cmathfuns,	6,	1,	1,	PP_FUNCALL},
 
+
 /* Mathematical Functions of Three Variables */
 
 {"dbeta",	do_math3,	1,	11,	3,	PP_FUNCALL},
@@ -386,7 +387,9 @@ FUNTAB R_FunTab[] =
 
 {"rhyper",	do_random3,	0,	11,	4,	PP_FUNCALL},
 
-{"sample",	do_sample,	0,	11,	4,	PP_FUNCALL},{"RNGkind",	do_RNGkind,	0,	11,	1,	PP_FUNCALL},
+{"sample",	do_sample,	0,	11,	4,	PP_FUNCALL},
+
+{"RNGkind",	do_RNGkind,	0,	11,	1,	PP_FUNCALL},
 
 /* Data Summaries */
 
@@ -427,6 +430,7 @@ FUNTAB R_FunTab[] =
 {"grep",	do_grep,	1,	11,	5,	PP_FUNCALL},
 {"sub",		do_gsub,	0,	11,	5,	PP_FUNCALL},
 {"gsub",	do_gsub,	1,	11,	5,	PP_FUNCALL},
+{"regexpr",	do_regexpr,	1,	11,	3,	PP_FUNCALL},
 
 
 /* Type Checking */
@@ -498,7 +502,8 @@ FUNTAB R_FunTab[] =
 {"flush.console",do_flushconsole,0,     11,     0,      PP_FUNCALL},
 {"int.unzip",   do_int_unzip,   0,      11,    -1,      PP_FUNCALL},
 {"win.version", do_winver,      0,      11,     0,      PP_FUNCALL},
-{"saveDevga", do_saveDevga,     0,      11,     3,      PP_FUNCALL},
+{"saveDevga",  do_saveDevga,    0,      11,     3,      PP_FUNCALL},
+{"shell.exec",  do_shellexec,    0,      11,     1,      PP_FUNCALL},
 #endif
 {"parse",	do_parse,	0,	11,	4,	PP_FUNCALL},
 {"save",	do_save,	0,	111,	3,	PP_FUNCALL},
@@ -509,7 +514,7 @@ FUNTAB R_FunTab[] =
 {"dput",	do_dput,	0,	111,	2,	PP_FUNCALL},
 {"dump",	do_dump,	0,	111,	2,	PP_FUNCALL},
 {"substitute",	do_substitute,	0,	0,	-1,	PP_FUNCALL},
-{"quit",	do_quit,	0,	111,	1,	PP_FUNCALL},
+{"quit",	do_quit,	0,	111,	3,	PP_FUNCALL},
 {"interactive",	do_interactive,	0,	0,	0,	PP_FUNCALL},
 {"readline",	do_readln,	0,	11,	1,	PP_FUNCALL},
 {"menu",	do_menu,	0,	11,	1,	PP_FUNCALL},
@@ -685,7 +690,7 @@ SEXP do_primitive(SEXP call, SEXP op, SEXP args, SEXP env)
     checkArity(op, args);
     name = CAR(args);
     if (!isString(name) || length(name) < 1 || STRING(name)[0] == R_NilValue)
-	errorcall(call, "string argument required\n");
+	errorcall(call, "string argument required");
     for (i = 0; R_FunTab[i].name; i++)
 	if (strcmp(CHAR(STRING(name)[0]), R_FunTab[i].name) == 0) {
 	    if ((R_FunTab[i].eval % 100 )/10)
@@ -693,7 +698,7 @@ SEXP do_primitive(SEXP call, SEXP op, SEXP args, SEXP env)
 	    else
 		return mkPRIMSXP(i, R_FunTab[i].eval % 10);
 	}
-    errorcall(call, "no such primitive function\n");
+    errorcall(call, "no such primitive function");
     return(R_NilValue);		/* -Wall */
 }
 
@@ -813,9 +818,9 @@ SEXP install(char *name)
     int i;
 
     if (*name == '\0')
-	error("attempt to use zero-length variable name\n");
+	error("attempt to use zero-length variable name");
     if (strlen(name) > MAXIDSIZE)
-	error("symbol print-name too long\n");
+	error("symbol print-name too long");
     strcpy(buf, name);
     i = hashpjw(buf);
     /* Check to see if the symbol is already present. */
@@ -839,12 +844,12 @@ SEXP do_internal(SEXP call, SEXP op, SEXP args, SEXP env)
     checkArity(op, args);
     s = CAR(args);
     if (!isPairList(s))
-	errorcall(call, "invalid .Internal() argument\n");
+	errorcall(call, "invalid .Internal() argument");
     fun = CAR(s);
     if (!isSymbol(fun))
-	errorcall(call, "invalid internal function\n");
+	errorcall(call, "invalid internal function");
     if (INTERNAL(fun) == R_NilValue)
-	errorcall(call, "no internal function \"%s\"\n", CHAR(PRINTNAME(fun)));
+	errorcall(call, "no internal function \"%s\"", CHAR(PRINTNAME(fun)));
     args = CDR(s);
     if (TYPEOF(INTERNAL(fun)) == BUILTINSXP)
 	args = evalList(args, env);
@@ -853,7 +858,7 @@ SEXP do_internal(SEXP call, SEXP op, SEXP args, SEXP env)
     args = PRIMFUN(INTERNAL(fun)) (s, INTERNAL(fun), args, env);
     UNPROTECT(1);
     if (save != R_PPStackTop) {
-	REprintf("stack imbalance in internal %s, %d then %d\n",
+	REprintf("stack imbalance in internal %s, %d then %d",
 	       PRIMNAME(INTERNAL(fun)), save, R_PPStackTop);
     }
     return (args);
