@@ -302,7 +302,7 @@ static double TeX(TEXPAR which)
     case xi13:	  /* big_op_spacing5 */
 	return 0.15 * XHeight();
     default:/* never happens (enum type) */
-	error("invalid `which' in TeX()!\n"); return 0;/*-Wall*/
+	error("invalid `which' in TeX()!"); return 0;/*-Wall*/
     }
 }
 
@@ -349,7 +349,7 @@ static void SetStyle(STYLE newstyle)
 	MathDevice->gp.cex = 0.5 * BaseCex;
 	break;
     default:
-	error("invalid math style encountered\n");
+	error("invalid math style encountered");
     }
     CurrentStyle = newstyle;
 }
@@ -1210,7 +1210,7 @@ static BBOX RenderSpace(SEXP expr, int draw)
 	return opBBox;
     }
     else
-	error("invalid mathematical annotation\n");
+	error("invalid mathematical annotation");
 
     return NullBBox();		/* -Wall */
 }
@@ -1342,7 +1342,7 @@ static BBOX RenderBin(SEXP expr, int draw)
 	return CombineBBoxes(bbox, RenderElement(CADR(expr), draw));
     }
     else
-	error("invalid mathematical annotation\n");
+	error("invalid mathematical annotation");
 
     return NullBBox();		/* -Wall */
 
@@ -1620,7 +1620,7 @@ static int AccentAtom(SEXP expr)
 
 static void InvalidAccent(SEXP expr)
 {
-    errorcall(expr, "invalid accent\n");
+    errorcall(expr, "invalid accent");
 }
 
 static BBOX RenderAccent(SEXP expr, int draw)
@@ -1829,7 +1829,7 @@ static int DelimCode(SEXP expr, SEXP head)
 	    code = '.';
     }
     if (code == 0)
-	errorcall(expr, "invalid group delimiter\n");
+	errorcall(expr, "invalid group delimiter");
     return code;
 }
 
@@ -1854,7 +1854,7 @@ static BBOX RenderGroup(SEXP expr, int draw)
     BBOX bbox;
     int code;
     if (length(expr) != 4)
-	errorcall(expr, "invalid group specification\n");
+	errorcall(expr, "invalid group specification");
     bbox = NullBBox();
     code = DelimCode(expr, CADR(expr));
     MathDevice->gp.cex = DelimSymbolMag * MathDevice->gp.cex;
@@ -1923,7 +1923,7 @@ static BBOX RenderDelim(int which, double dist, int draw)
 	top = 252; ext = 239; bot = 254; mid = 253;
 	break;
     default:
-	error("group is incomplete\n");
+	error("group is incomplete");
 	return ansBBox;/*never reached*/
     }
     topBBox = GlyphBBox(top);
@@ -1996,7 +1996,7 @@ static BBOX RenderBGroup(SEXP expr, int draw)
     double extra = 0.2 * xHeight();
     int delim1, delim2;
     if (length(expr) != 4)
-	errorcall(expr, "invalid group specification\n");
+	errorcall(expr, "invalid group specification");
     bbox = NullBBox();
     delim1 = DelimCode(expr, CADR(expr));
     delim2 = DelimCode(expr, CADDDR(expr));
@@ -2474,7 +2474,7 @@ static BBOX RenderRel(SEXP expr, int draw)
 	bbox = CombineBBoxes(bbox, RenderGap(gap, draw));
 	return CombineBBoxes(bbox, RenderElement(CADDR(expr), draw));
     }
-    else error("invalid mathematical annotation\n");
+    else error("invalid mathematical annotation");
 
     return NullBBox();		/* -Wall */
 }
@@ -2887,7 +2887,7 @@ void GMathText(double x, double y, int coords, SEXP expr,
     double ascent, descent, width;
     GMetricInfo(0, &ascent, &descent, &width, DEVICE, dd);
     if ((ascent==0) && (descent==0) && (width==0))
-	error("Metric information not yet available for this device\n");
+	error("Metric information not yet available for this device");
 #endif
 
     MathDevice = dd;
@@ -2930,7 +2930,7 @@ void GMMathText(SEXP str, int side, double line, int outer,
     double ascent, descent, width;
     GMetricInfo(0, &ascent, &descent, &width, DEVICE, dd);
     if ((ascent==0) && (descent==0) && (width==0))
-	error("Metric information not yet available for this device\n");
+	error("Metric information not yet available for this device");
 #endif
 
     MathDevice = dd;
@@ -2938,27 +2938,25 @@ void GMMathText(SEXP str, int side, double line, int outer,
     if (outer) {
 	switch (side) {
 	case 1:
-	    line = line + 1;
-	    coords = MAR1;
+	    coords = OMA1;
 	    a = 0;
 	    xadj = MathDevice->gp.adj;
 	    yadj = NA_REAL;
 	    break;
 	case 2:
-	    coords = MAR2;
+	    coords = OMA2;
 	    a = 90;
 	    xadj = MathDevice->gp.adj;
 	    yadj = NA_REAL;
 	    break;
 	case 3:
-	    coords = MAR3;
+	    coords = OMA3;
 	    a = 0.0;
 	    xadj = MathDevice->gp.adj;
 	    yadj = NA_REAL;
 	    break;
 	case 4:
-	    line = line + 1;
-	    coords = MAR4;
+	    coords = OMA4;
 	    a = 90.0;
 	    xadj = MathDevice->gp.adj;
 	    yadj = NA_REAL;
