@@ -14,7 +14,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA.
  *
  *  SYNOPSIS
  *
@@ -56,7 +56,7 @@ double lbeta(double a, double b)
 	return ML_POSINF;
     }
 #ifdef IEEE_754
-    else if (!FINITE(q)) {
+    else if (!R_FINITE(q)) {
 	return ML_NEGINF;
     }
 #endif
