@@ -14,7 +14,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA.
  *
  *  SYNOPSIS
  *
@@ -42,7 +42,7 @@ double ppois(double x, double lambda)
     if (x < 0)
 	return 0;
 #ifdef IEEE_754
-    if (!finite(x))
+    if (!R_FINITE(x))
 	return 1;
 #endif
     return  1 - pgamma(lambda, x + 1, 1.0);
