@@ -9,7 +9,7 @@ aroot <- substring(as.bi <- bi[substring(bi,1,3) == "as."],4)
 ##--- producing the real R script:
 sink("isas-tests.R")
 
-cat("options(error.halt = FALSE)",
+cat("options(error = expression(NULL))",
     "# don't stop on error in batch\n##~~~~~~~~~~~~~~\n")
 for(x in expression(1, NULL, pi, "1.3", list(), list(a=1))) {
     cat("\n###--------\n x <- ", deparse(x), "\n", sep="")
